@@ -1,8 +1,27 @@
 package DBIx::Librarian::Statement;
 
-require 5.004;
+require 5.005;
 use strict;
+use base qw(Class::Virtual);
+use vars qw($VERSION);
+$VERSION = '0.4';
+
 use Carp;
+use Time::HiRes qw( gettimeofday tv_interval );
+
+use Log::Channel;
+{
+    my $execlog = new Log::Channel "exec";
+    sub execlog { $execlog->(@_) }
+    my $perflog = new Log::Channel "perf";
+    sub perflog { $perflog->(@_) }
+    my $rowlog = new Log::Channel "rows";
+    sub rowlog { $rowlog->(@_) }
+    my $bindlog = new Log::Channel "bind";
+    sub bindlog { $bindlog->(@_) }
+}
+
+__PACKAGE__->virtual_methods(qw(fetch));
 
 =head1 NAME
 
@@ -66,9 +85,9 @@ sub new {
     #   Yuck.  Is there a cross-platform way to do this?
 
 
-    my @bindvars = $sql =~ /[^A-Za-z0-9:]:(\w+)/mog;
+    my @bindvars = $sql =~ /[^A-Za-z0-9:]:([[:word:].]+)/mog;
     if (@bindvars) {
-	$sql =~ s/([^A-Za-z0-9:]):\w+/$1?/og;
+	$sql =~ s/([^A-Za-z0-9:]):[[:word:].]+/$1?/og;
     }
     $self->{BINDVARS} = \@bindvars;
 
@@ -99,13 +118,15 @@ sub new {
 	# requires on-demand parsing
 	@directvars = $sql =~ /\$(\w+)/og;
 	$self->{DIRECTVARS} = \@directvars;
-	$self->{SQL} = $sql;
+#	$self->{SQL} = $sql;
     }
     else {
 	# can prepare in advance
 	$self->_prepare($sql);
     }
 
+    $self->{SQL} = $sql;
+
     return $self;
 }
 
@@ -113,8 +134,7 @@ sub new {
 sub _prepare {
     my ($self, $sql) = @_;
 
-    print STDERR "PREPARE SQL:\n", $sql, "\n====================\n"
-      if $self->{TRACE};
+    execlog "PREPARE SQL:\n", $sql, "\n====================\n";
 
     my $sth = $self->{DBH}->prepare($sql);
     if (!$sth) {
@@ -137,25 +157,41 @@ Croaks on any database error or if any SELECT criteria are violated.
 sub execute {
     my ($self, $data) = @_;
 
+    execlog "EXECUTE SQL:\n", $self->{SQL}, "\n==========\n";
+
     $self->_substitutions($data);
 
     my @bindlist = $self->_bind($data);
 
     if ($self->{IS_SELECT} && !$data) {
-	croak "Missing target data reference in SQL\n$self->{STH}->{Statement}\n";
+	# caller(0) gives this line here in Statement.pm
+	# caller(1) gives line in _execute() in Librarian.pm
+	# caller(2) gives line in execute() in Librarian.pm
+	# caller(3) gives the application code invoking Librarian::execute()
+	my (undef, $filename, $line) = caller(3);
+	croak "Missing required data parameter for SQL execute(\"$self->{NAME}\")\n at $filename line $line\n";
     }
 
+    my $t0 = [ gettimeofday ];
+
     if (! $self->{STH}->execute(@bindlist)) {
 	croak $self->{DBH}->errstr. " in SQL\n$self->{STH}->{Statement}\n";
     }
 
+    my $elapsed = tv_interval ($t0, [ gettimeofday ]);
+    perflog "ELAPSED TIME: ", $elapsed, "\n";
+
+    my $rows;
     if ($self->{IS_SELECT}) {
-	return $self->fetch($data);
-#	return 0;
+	$rows = $self->fetch($data);
+	rowlog "ROWS RETRIEVED: ", $rows, "\n";
     }
     else {
-	return $self->{STH}->rows;
+	rowlog "ROWS ALTERED: ", $self->{STH}->rows, "\n";
+	$rows = $self->{STH}->rows;
     }
+
+    return $rows;
 }
 
 
@@ -176,9 +212,9 @@ sub _substitutions {
 	    croak "Expected scalar for $directvar" if ref($data->{$directvar});
 	    $val = $data->{$directvar};
 	}
-	printf STDERR ("\tSUB \$%s = %s\n",
-		       $directvar,
-		       $val || '(null)') if $self->{TRACE};
+	bindlog sprintf ("\tSUB \$%s = %s\n",
+			 $directvar,
+			 $val || '(null)');
 	$sql =~ s/\$$directvar(\W|$)/$val$1/g;
     }
 
@@ -187,8 +223,6 @@ sub _substitutions {
 	croak $self->{DBH}->errstr . " in SQL\n$sql\n";
     }
     $self->{STH} = $sth;
-
-#    print STDERR "SUBSITUTION COMPLETE:\n", $sql, "\n";
 }
 
 
@@ -205,16 +239,25 @@ sub _bind {
 
     foreach my $bindvar (@{$self->{BINDVARS}}) {
 	my $val;
+	my $node = $data;
+	my $key = $bindvar;
+	while ($key =~ /\./) {
+	    # drill down into the hierarchy as needed to find sub-elements
+	    my ($base, $subkey) = $key =~ /(.*?)\.(.*)/;
+	    croak "Expected scalar for $key" if !defined $node->{$base} || !ref($node->{$base});
+	    $node = $node->{$base};
+	    $key = $subkey;
+	}
 	if ($self->{ALLARRAYS}) {
-	    $val = $data->{$bindvar}[0];
+	    $val = $node->{$key}[0];
 	} else {
-	    croak "Expected scalar for $bindvar" if ref($data->{$bindvar});
-	    $val = $data->{$bindvar};
+	    croak "Expected scalar for $key" if ref($data->{$key});
+	    $val = $node->{$key};
 	}
 
-	printf STDERR ("\tBIND :%s = %s\n",
-		       $bindvar,
-		       $val || '(null)') if $self->{TRACE};
+	bindlog sprintf ("\tBIND :%s = %s\n",
+			 $bindvar,
+			 $val || '(null)');
 	push @bindlist, $val;
     }
 
@@ -229,6 +272,8 @@ Jason W. May <jmay@pobox.com>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2001 Jason W. May.  All rights reserved.
+Copyright (C) 2001-2003 Jason W. May.  All rights reserved.
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
+
+=cut
